@@ -10,6 +10,7 @@ import sys
 import typing as t
 from functools import lru_cache
 from functools import partial
+from urllib.parse import urlsplit
 
 import anyio.to_thread
 import attrs
@@ -138,6 +139,9 @@ class Service(t.Generic[T]):
     def __hash__(self):
         return hash(self.name)
 
+    def has_custom_command(self) -> bool:
+        return hasattr(self.inner, "__command__") or self.cmd is not None
+
     @_caller_module.default  # type: ignore
     def _get_caller_module(self) -> str:
         if __name__ == "__main__":
@@ -184,6 +188,40 @@ class Service(t.Generic[T]):
         url = dependency_map.get(self.name)
         return url.replace("tcp://", "http://") if url else None
 
+    async def get_hosts(self) -> list[str]:
+        """Return a list of IPs of the service"""
+        import httpx
+
+        from _bentoml_impl.server.allocator import ResourceAllocator
+
+        url = BentoMLContainer.remote_runner_mapping.get().get(self.name)
+        if not url:
+            raise BentoMLException(f"Service {self.name} not found")
+        url = url.replace("tcp://", "http://")
+        workers, _ = ResourceAllocator().get_worker_env(self)
+        if url.startswith("file://"):
+            # UDS connections don't have a port number, use a fake one.
+            return ["127.0.0.1:3000"] * workers
+        if not url.startswith("http://"):
+            raise BentoMLException(
+                f"Unable to get hosts for service {self.name} because it is not served as HTTP"
+            )
+        if "BENTOCLOUD_DEPLOYMENT_URL" in os.environ:
+            # BentoCloud environment, the url is to runner-lb
+            headers = {"Runner-Name": self.name, "Resolve-Runner": "1"}
+            async with httpx.AsyncClient() as client:
+                response = await client.get(url, headers=headers)
+                if response.is_error:
+                    raise BentoMLException(
+                        f"Failed to get hosts for service {self.name} from cloud"
+                    )
+                await response.aread()
+                result = response.json()
+                return [f"{ip}:{result['port']}" for ip in result.get("ips", [])]
+
+        # Serving locally, the hostname should be the IP
+        return [urlsplit(url).netloc] * workers
+
     def all_services(self, exclude_urls: bool = False) -> dict[str, Service[t.Any]]:
         """Get a map of the service and all recursive dependencies"""
         services: dict[str, Service[t.Any]] = {self.name: self}
