@@ -1,43 +1,31 @@
 from __future__ import annotations
 
-import os
-import re
-import sys
-import random
-import socket
-import typing as t
+import asyncio
+import contextlib
+import functools
 import inspect
 import logging
-import functools
-import contextlib
-from typing import overload
-from typing import TYPE_CHECKING
-from pathlib import Path
-from reprlib import recursive_repr as _recursive_repr
+import os
+import random
+import re
+import socket
+import typing as t
+import warnings
 from datetime import date
-from datetime import time
 from datetime import datetime
+from datetime import time
 from datetime import timedelta
+from reprlib import recursive_repr as _recursive_repr
+from typing import TYPE_CHECKING
+from typing import overload
 
-import fs
-import attr
-import fs.copy
-from rich.console import Console
-
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from backports.cached_property import cached_property
-
-from .cattr import bentoml_cattr
 from ..types import LazyType
-from .lazy_loader import LazyLoader
 
 if TYPE_CHECKING:
-    from fs.base import FS
+    import attr
+    from starlette.applications import Starlette
     from typing_extensions import Self
 
-    from ..types import PathType
     from ..types import MetadataDict
     from ..types import MetadataType
 
@@ -47,22 +35,7 @@ if TYPE_CHECKING:
 
 C = t.TypeVar("C")
 T = t.TypeVar("T")
-_T_co = t.TypeVar("_T_co", covariant=True, bound=t.Any)
 
-rich_console = Console(theme=None)
-
-__all__ = [
-    "bentoml_cattr",
-    "cached_property",
-    "cached_contextmanager",
-    "reserve_free_port",
-    "LazyLoader",
-    "validate_or_create_dir",
-    "display_path_under_home",
-    "rich_console",
-    "experimental",
-    "compose",
-]
 
 _EXPERIMENTAL_APIS: set[str] = set()
 
@@ -121,13 +94,11 @@ def add_experimental_docstring(f: t.Callable[P, t.Any]) -> t.Callable[P, t.Any]:
 
 
 @overload
-def first_not_none(*args: T | None, default: T) -> T:
-    ...
+def first_not_none(*args: T | None, default: T) -> T: ...
 
 
 @overload
-def first_not_none(*args: T | None) -> T | None:
-    ...
+def first_not_none(*args: T | None) -> T | None: ...
 
 
 def first_not_none(*args: T | None, default: None | T = None) -> T | None:
@@ -147,31 +118,6 @@ def normalize_labels_value(label: dict[str, t.Any] | None) -> dict[str, str] | N
     return {k: str(v) for k, v in label.items()}
 
 
-def validate_or_create_dir(*path: PathType) -> None:
-    for p in path:
-        path_obj = Path(p)
-
-        if path_obj.exists():
-            if not path_obj.is_dir():
-                raise OSError(20, f"{path_obj} is not a directory")
-        else:
-            path_obj.mkdir(parents=True)
-
-
-def calc_dir_size(path: PathType) -> int:
-    return sum(f.stat().st_size for f in Path(path).glob("**/*") if f.is_file())
-
-
-def display_path_under_home(path: str) -> str:
-    # Shorten path under home directory with leading `~`
-    # e.g. from `/Users/foo/bar` to just `~/bar`
-    try:
-        return str("~" / Path(path).relative_to(Path.home()))
-    except ValueError:
-        # when path is not under home directory, return original full path
-        return path
-
-
 def human_readable_size(size: t.Union[int, float], decimal_places: int = 2) -> str:
     for unit in ["B", "KiB", "MiB", "GiB", "TiB", "PiB"]:
         if size < 1024.0 or unit == "PiB":
@@ -194,9 +140,9 @@ def split_with_quotes(
     ['a', 'b', 'c,d', 'e']
     """
     if use_regex:
-        assert (
-            "(" not in sep and ")" not in sep
-        ), "sep cannot contain '(' or ')' when using regex"
+        assert "(" not in sep and ")" not in sep, (
+            "sep cannot contain '(' or ')' when using regex"
+        )
         reg = "({quote}[^{quote}]*{quote})|({sep})".format(
             quote=quote,
             sep=sep,
@@ -270,46 +216,6 @@ def reserve_free_port(
         sock.close()
 
 
-def copy_file_to_fs_folder(
-    src_path: str,
-    dst_fs: FS,
-    dst_folder_path: str = ".",
-    dst_filename: t.Optional[str] = None,
-):
-    """Copy the given file at src_path to dst_fs filesystem, under its dst_folder_path
-    folder with dst_filename as file name. When dst_filename is None, keep the original
-    file name.
-    """
-    src_path = os.path.realpath(os.path.expanduser(src_path))
-    dir_name, file_name = os.path.split(src_path)
-    src_fs = fs.open_fs(dir_name)
-    dst_filename = file_name if dst_filename is None else dst_filename
-    dst_path = fs.path.join(dst_folder_path, dst_filename)
-    dst_fs.makedir(dst_folder_path, recreate=True)
-    fs.copy.copy_file(src_fs, file_name, dst_fs, dst_path)
-
-
-def resolve_user_filepath(filepath: str, ctx: t.Optional[str]) -> str:
-    """Resolve the abspath of a filepath provided by user. User provided file path can:
-    * be a relative path base on ctx dir
-    * contain leading "~" for HOME directory
-    * contain environment variables such as "$HOME/workspace"
-    """
-    # Return if filepath exist after expanduser
-
-    _path = os.path.expanduser(os.path.expandvars(filepath))
-    if os.path.exists(_path):
-        return os.path.realpath(_path)
-
-    # Try finding file in ctx if provided
-    if ctx:
-        _path = os.path.expanduser(os.path.join(ctx, filepath))
-        if os.path.exists(_path):
-            return os.path.realpath(_path)
-
-    raise FileNotFoundError(f"file {filepath} not found")
-
-
 def label_validator(
     _: t.Any, _attr: attr.Attribute[dict[str, str]], labels: dict[str, str]
 ):
@@ -496,3 +402,145 @@ class compose:
     def functions(self):
         """Read-only tuple of the composed callables, in order of execution."""
         return (self.__wrapped__,) + tuple(self._wrappers)
+
+
+def get_original_func(obj: t.Callable[..., t.Any]) -> t.Callable[..., t.Any]:
+    """
+    Get the original function from a decorated function
+    """
+    if getattr(obj, "__is_bentoml_api_func__", False):
+        obj = obj.func  # type: ignore
+    while isinstance(obj, functools.partial):
+        obj = obj.func
+    return obj
+
+
+def is_async_callable(obj: t.Any) -> t.TypeGuard[t.Callable[..., t.Awaitable[t.Any]]]:
+    obj = get_original_func(obj)
+
+    return asyncio.iscoroutinefunction(obj) or (
+        callable(obj) and asyncio.iscoroutinefunction(obj.__call__)
+    )
+
+
+def dict_filter_none(d: dict[str, t.Any]) -> dict[str, t.Any]:
+    return {k: v for k, v in d.items() if v is not None}
+
+
+CONTROL_CODES = re.compile(
+    r"""
+    \x07|                  # BELL
+    \r|                    # CARRIAGE_RETURN
+    \x1b\[H|               # HOME
+    \x1b\[2J|              # CLEAR
+    \x1b\[?1049h|          # ENABLE_ALT_SCREEN
+    \x1b\[?1049l|          # DISABLE_ALT_SCREEN
+    \x1b\[?25h|            # SHOW_CURSOR
+    \x1b\[?25l|            # HIDE_CURSOR
+    \x1b\[\d+A|            # CURSOR_UP
+    \x1b\[\d+B|            # CURSOR_DOWN
+    \x1b\[\d+C|            # CURSOR_FORWARD
+    \x1b\[\d+D|            # CURSOR_BACKWARD
+    \x1b\[\d+G|            # CURSOR_MOVE_TO_COLUMN
+    \x1b\[\d+K|            # ERASE_IN_LINE
+    \x1b\[\d+;\d+H|        # CURSOR_MOVE_TO
+    \x1b\]0;.+?\x07        # SET_WINDOW_TITLE
+""",
+    flags=re.VERBOSE,
+)
+
+
+def filter_control_codes(line: str) -> str:
+    return CONTROL_CODES.sub("", line)
+
+
+def with_app_arg(func: t.Callable[[], T]) -> t.Callable[[Starlette], T]:
+    @functools.wraps(func)
+    def wrapper(app: Starlette) -> T:
+        return func()
+
+    return wrapper
+
+
+class BentoMLDeprecationWarning(DeprecationWarning):
+    pass
+
+
+warnings.simplefilter("default", BentoMLDeprecationWarning)
+
+
+def warn_deprecated(message: str, stacklevel: int = 2) -> None:
+    warnings.warn(
+        message, category=BentoMLDeprecationWarning, stacklevel=stacklevel + 1
+    )
+
+
+def deprecated(
+    name: str = "", deprecated_since: str = "1.4", suggestion: str = ""
+) -> t.Callable[[C], C]:
+    def decorator(func: t.Callable[P, T]) -> t.Callable[P, T]:
+        obj_name = name or func.__name__
+
+        class _DeprecatedMixin:
+            def __init__(self, *args: P.args, **kwargs: P.kwargs) -> None:
+                warn_deprecated(
+                    f"`{obj_name}` is deprecated since BentoML v{deprecated_since} and will be removed in a future version."
+                    + (f" {suggestion}" if suggestion else "")
+                )
+                super().__init__(*args, **kwargs)
+
+        if inspect.isclass(func):
+            return type(func.__name__, (_DeprecatedMixin, func), {})
+
+        @functools.wraps(func)
+        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
+            warn_deprecated(
+                f"`{obj_name}` is deprecated since BentoML v{deprecated_since} and will be removed in a future version."
+                + (f" {suggestion}" if suggestion else "")
+            )
+            return func(*args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
+def is_jupyter() -> bool:  # pragma: no cover
+    """Check if we're running in a Jupyter notebook."""
+    try:
+        get_ipython  # type: ignore[name-defined]
+    except NameError:
+        return False
+    ipython = get_ipython()  # noqa: F821 # type: ignore[name-defined]
+    shell = ipython.__class__.__name__
+    if (
+        "google.colab" in str(ipython.__class__)
+        or os.getenv("DATABRICKS_RUNTIME_VERSION")
+        or shell == "ZMQInteractiveShell"
+    ):
+        return True  # Jupyter notebook or qtconsole
+    elif shell == "TerminalInteractiveShell":
+        return False  # Terminal running IPython
+    else:
+        return False  # Other type (?)
+
+
+def deep_merge(base: t.Any, update: t.Any) -> dict[t.Any, t.Any]:
+    """Merge two dictionaries recursively in place. The base dict will be updated."""
+    if not isinstance(base, dict) or not isinstance(update, dict):
+        return update
+    for k, v in update.items():
+        if k in base and isinstance(base[k], dict) and isinstance(v, dict):
+            base[k] = deep_merge(base[k], v)
+        else:
+            base[k] = v
+    return base
+
+
+def expand_envs(value: str, envs: t.Mapping[str, str] | None = None) -> str:
+    """Expand environment variables in a string."""
+    import string
+
+    if envs is None:
+        envs = os.environ
+    return string.Template(value).substitute(envs)
