@@ -1,31 +1,30 @@
 from __future__ import annotations
 
-import typing as t
 import importlib
 import traceback
-from typing import TYPE_CHECKING
+import typing as t
 from contextlib import ExitStack
 from contextlib import asynccontextmanager
+from typing import TYPE_CHECKING
 
-from ...exceptions import BentoMLException
-from ...grpc.utils import import_grpc
-from ...grpc.utils import import_generated_stubs
-from ...grpc.utils import LATEST_PROTOCOL_VERSION
-from ..._internal.utils import LazyLoader
-from ..._internal.utils import reserve_free_port
 from ..._internal.utils import cached_contextmanager
-from ..._internal.utils import add_experimental_docstring
+from ..._internal.utils import reserve_free_port
+from ..._internal.utils.lazy_loader import LazyLoader
+from ...exceptions import BentoMLException
+from ...grpc.utils import LATEST_PROTOCOL_VERSION
+from ...grpc.utils import import_generated_stubs
+from ...grpc.utils import import_grpc
 
 if TYPE_CHECKING:
     import grpc
     import numpy as np
-    from grpc import aio
-    from numpy.typing import NDArray
-    from grpc.aio._channel import Channel
     from google.protobuf.message import Message
+    from grpc import aio
+    from grpc.aio._channel import Channel
+    from numpy.typing import NDArray
 
-    from ...grpc.v1 import service_pb2 as pb
     from ..._internal.service import Service
+    from ...grpc.v1 import service_pb2 as pb
 else:
     grpc, aio = import_grpc()  # pylint: disable=E1111
     np = LazyLoader("np", globals(), "numpy")
@@ -146,22 +145,21 @@ async def async_client_call(
         details = call.details()
         trailing_metadata = call.trailing_metadata()
     if assert_code is not None:
-        assert (
-            return_code == assert_code
-        ), f"Method '{method}' returns {return_code} while expecting {assert_code}."
+        assert return_code == assert_code, (
+            f"Method '{method}' returns {return_code} while expecting {assert_code}."
+        )
     if assert_details is not None:
-        assert (
-            assert_details == details
-        ), f"Details '{assert_details}' is not in '{details}'."
+        assert assert_details == details, (
+            f"Details '{assert_details}' is not in '{details}'."
+        )
     if assert_trailing_metadata is not None:
-        assert (
-            trailing_metadata == assert_trailing_metadata
-        ), f"Trailing metadata '{trailing_metadata}' while expecting '{assert_trailing_metadata}'."
+        assert trailing_metadata == assert_trailing_metadata, (
+            f"Trailing metadata '{trailing_metadata}' while expecting '{assert_trailing_metadata}'."
+        )
     return res
 
 
 @asynccontextmanager
-@add_experimental_docstring
 async def create_channel(
     host_url: str,
     interceptors: t.Sequence[aio.ClientInterceptor] | None = None,
@@ -190,7 +188,6 @@ async def create_channel(
             await channel.close()
 
 
-@add_experimental_docstring
 @cached_contextmanager("{interceptors}")
 def make_standalone_server(
     interceptors: t.Sequence[aio.ServerInterceptor] | None = None,
