@@ -1,8 +1,8 @@
 from __future__ import annotations
 
+import json
 import os
 import sys
-import json
 import typing as t
 
 import click
@@ -40,9 +40,9 @@ import click
     help="The environment variables to pass to the worker process. The format is a JSON string, e.g. '{0: {\"CUDA_VISIBLE_DEVICES\": 0}}'.",
 )
 @click.option(
-    "--prometheus-dir",
-    type=click.Path(exists=True),
-    help="Required by prometheus to pass the metrics in multi-process mode",
+    "--timeout",
+    type=click.INT,
+    help="Specify the timeout (seconds) for runner",
 )
 def main(
     bento_identifier: str,
@@ -51,8 +51,8 @@ def main(
     working_dir: t.Optional[str],
     no_access_log: bool,
     worker_id: int,
+    timeout: int | None,
     worker_env_map: str | None,
-    prometheus_dir: str | None,
 ) -> None:
     """
     Start a runner server.
@@ -67,15 +67,15 @@ def main(
     """
 
     # setting up the environment for the worker process
-    assert (
-        "bentoml" not in sys.modules
-    ), "bentoml should not be imported before setting up the environment, otherwise some of the environment may not take effect"
+    assert "bentoml" not in sys.modules, (
+        "bentoml should not be imported before setting up the environment, otherwise some of the environment may not take effect"
+    )
     if worker_env_map is not None:
         env_map: dict[str, dict[str, t.Any]] = json.loads(worker_env_map)
         worker_key = str(worker_id - 1)  # the worker ID is 1-based
-        assert (
-            worker_key in env_map
-        ), f"worker_id {repr(worker_key)} not found in worker_env_map: {worker_env_map}"
+        assert worker_key in env_map, (
+            f"worker_id {repr(worker_key)} not found in worker_env_map: {worker_env_map}"
+        )
         os.environ.update(env_map[worker_key])
 
     import socket
@@ -83,12 +83,12 @@ def main(
     import psutil
 
     from bentoml import load
-    from bentoml._internal.context import component_context
+    from bentoml._internal.context import server_context
 
     # setup context
-    component_context.component_type = "runner"
-    component_context.component_name = runner_name
-    component_context.component_index = worker_id
+    server_context.service_type = "runner"
+    server_context.service_name = runner_name
+    server_context.worker_index = worker_id
 
     from bentoml._internal.log import configure_server_logging
 
@@ -98,24 +98,21 @@ def main(
 
     from bentoml._internal.configuration.containers import BentoMLContainer
 
-    if prometheus_dir is not None:
-        BentoMLContainer.prometheus_multiproc_dir.set(prometheus_dir)
-
     if no_access_log:
         access_log_config = BentoMLContainer.runners_config.logging.access
         access_log_config.enabled.set(False)
 
     from bentoml._internal.server.runner_app import RunnerAppFactory
 
-    service = load(bento_identifier, working_dir=working_dir, standalone_load=True)
+    service = load(bento_identifier, working_dir=working_dir)
 
     # setup context
     if service.tag is None:
-        component_context.bento_name = service.name
-        component_context.bento_version = "not available"
+        server_context.bento_name = service.name
+        server_context.bento_version = "not available"
     else:
-        component_context.bento_name = service.tag.name
-        component_context.bento_version = service.tag.version or "not available"
+        server_context.bento_name = service.tag.name
+        server_context.bento_version = service.tag.version or "not available"
 
     for runner in service.runners:
         if runner.name == runner_name:
@@ -123,7 +120,10 @@ def main(
     else:
         raise ValueError(f"Runner {runner_name} not found")
 
-    app = RunnerAppFactory(runner, worker_index=worker_id)()
+    factory = RunnerAppFactory(runner, worker_index=worker_id)
+    if timeout is not None:
+        factory.timeout = timeout
+    app = factory()
 
     uvicorn_options: dict[str, int | None | str] = {
         "log_config": None,
